################################################################################
#                    www.PowerShell-Scripting.com
#
#                      PowerShell Form Converter
#
# Version : 0.10
#
# Statut  : A valider
#
# Date    : 16 avril 2008
#
# Nom     : Convert-Form.ps1
#
# Usage   : Convert-Form.ps1 -Source Form1.Designer.cs -Destination Form1.ps1 [-DontShow] [-DontLoad] [-Confirm]
#           -DontShow : n'insre pas, dans le fichier gnr, l'appel  ShowDialog() 
#			-DontLoad : n'insre pas, dans le fichier gnr, les appels aux assemblies Winform, etc. 
#						Cas d'une fentre secondaire n'utilisant pas d'assemblies spcifique (cas le plus probable)
#						C'est le script d'appel de la fentre principale qui les chargera. Il s'occupera galement 
#						de la cration, de l'affichage et de la destruction des fentres secondaires.
#			-Confirm  : demande la confirmation si le fichier cible(.ps1) existe dj
#
# Objet   : Conversion d'un formulaire graphique C# (form) cr  partir 
#           de Visual C# 2005 (Express Edition ou > ) en un script PowerShell.
# 
# Composants supports :  voir le fichier ..\Documentation\ComposantsSupportes.txt
#
# Composants interdits :  
#                       -System.Windows.Forms.WebBrowser : Limite de PowerShell v1.0
#                            cf. :  http://blogs.msdn.com/powershell/archive/2007/03/23/thread-apartmentstate-and-powershell-execution-thread.aspx
#						-Le composant backGroundWorker mme raison que pour le WebBrowser.Bien que sont convertion/cratio nne pose pas de pb.
#						 Il ne dipose pas de RunSpace pour excuter des commandes PS. 
#						 En v1 utiliser  la place un 'job'.
#						  voir "Background Jobs and PowerShell"  --> http://www.leeholmes.com/blog/RealtimeSyntaxHighlightingInYourPowerShellConsole.aspx
# Composants en erreur :  
# 						-TableLayoutPanel la cration imbriqu de composants n'est pas encore supporte
#						-La reconnaissance de certaines combinaisons de caractres n'est pas prise en compte.
#
# Limitations :
#	   -L'accessibilit n'est pas supporte (test)
#	   -Le databinding n'est pas support	(non-test)
#	   -La gestion de rfrences d'assemblies autres que Winform, Drawing,Resources n'est pas support (test).
#		 voir dans le fichier projet C# (XML), le noeud :
#			<ItemGroup>
#			  <Reference Include="CrystalDecisions.CrystalReports.Engine, Version= ...
#
# Informations :
#				L'expression rgulire Match n'est pas sensible  la casse lors de la recherche
#				Pensez  nommer chaque composants Winform d'aprs le nom de votre projet.
#				 Ex FrmMainProjet1,FrmSecondaireProjet1
#
#       Sous Powershell la dclaration simultane des vnements Activated et DeActivated, ralise pour VisualStudio, peut ammener le formulaire " boucler".
#       Idem pour la dclaration de l'vnements Paint, de la dclaration des vnements de la souris, etc, etc.
#       Une gestion correcte du code de ces gestionnaires, en lieu et place d'un appel  MessageBox, rglera le problme.
#
#############################################################################################################
Param([string] $Source, [string] $Destination, [switch] $DontShow, [switch] $DontLoad, [Switch] $Confirm)

#-------------------- Contrles divers ----------------------------------------------------------------------
if ((!$Source) -or (!$Destination)) 
   { Write-Host "Usage :`n`r CSForm2PS.ps1 -Source Form1.Designer.cs -Destination Form1.ps1 [-DontShow] [-DontLoad] [-Confirm]`n`r"
     Write-Host "  Le switch -DontShow spcifie la suppression,  la fin du script PS1 gnr, de l'appel  la mthode ShowDialog()."
     Write-Host "  Le switch -DontLoad spcifie de ne pas insrer, dans le fichier gnr, les appels aux assemblies Winform, etc."
     Write-Host "  Le switch -Confirm  spcifie une demande de confirmation si le fichier cible (.ps1) existe dj.`n`r"
     Write-Host " Codes couleur :"
     Write-Host "`tInformation."
     Write-Host -noNewLine "`tInformation: ";Write-Host "opration russie." -f Green
     Write-Host -noNewLine "`tInformation: ";Write-Host "opration en chec" -f DarkYellow 
     Write-Host -noNewLine "`tErreur: ";Write-Host "non-bloquante." -f Yellow 
     Write-Host -noNewLine "`tErreur: ";Write-Host "grave." -f red 
     Return
   }   

if ( !(Test-Path $Source))
 {
  Write-Warning "Le fichier source n'existe pas :`n`r $Source"
  break;
 }
#Teste l'existence des scripts ncessaires
#Puis les charges

function Get-ScriptDirectory
{  #Renvoi le nom du rpertoire d'un script parent, celui appel sur la ligne de commande.
   # By J.Snoover
  $Invocation = (Get-Variable MyInvocation -Scope 1).Value
  Split-Path $Invocation.MyCommand.Path
}

$ScriptPath = Get-ScriptDirectory

$PckScripts = Join-Path $ScriptPath PackageScripts.ps1
if ( !(Test-Path $PckScripts))
 {
  Write-Warning "Le script suivant n'existe pas :`n`r $PckScripts"
  break;
 }
. $PckScripts

$PckConvertForm = Join-Path $ScriptPath PackageConvert-Form.ps1
TestRequiredScripts $PckConvertForm
 #TestRequiredCommands aucune
 #TestRequiredFunctions aucune

. $PckConvertForm
MakeFilesName

if (($Confirm.IsPresent) -and (Test-Path $Destination))
 {  
  Write-Host "Le fichier de destination existe dj, voulez-vous le supprimer ?`n`r$Destination"
  $Continuer =$True
  $Rponse = Read-Host "[O] Oui [N] Non"
  while ($Continuer) 
   {
    Switch ($rponse)
    {
     "O" {$Continuer =$false}
     "N" {Write-Warning "Opration abandonne."; exit}
     default {$rponse = Read-Host "[O] Oui [N] Non"}
    }
   }
 }
Write-Debug "Fin des contrles."
function Finalyze
{
  #Libration des listes
  if ($LinesNewScript -ne $Null)
   {
    $LinesNewScript.Clear()
    $LinesNewScript=$null
   }
  if ($Components -ne $Null)
   {
    $Components.Clear()
  	$Components=$null
   }
}
 
#-------------------- Fin des Contrles --------------------------------------------------------------------

# Collection des lignes utiles de InitializeComponent() : $Components
# Note:
# Le code gnr automatiquement par le concepteur Windows Form est insr 
# dans la mthode InitializeComponent. 
# L'intgralit du code d'une mthode C# est dlimit par { ... } 
# On insre ces lignes de code et uniquement celles-ci dans le tableau $Component.
# -----------------------------------------------------------------------------
$Components = New-Object System.Collections.ArrayList(400)
[boolean]  $isDebutCodeInit = $false
[string]   $FormName = ""

foreach ($ligne in (get-content $Source))
{
 if (! $isDebutCodeInit)
    {  # On dmarre l'insertion  partir de cette ligne
       # On peut donc suppos que l'on parse un fichier cre par le designer VS
      if ($Ligne.contains("InitializeComponent()")) {$isDebutCodeInit= $true}
    }
 else 
    {  # Fin de la mthode rencontre ou ligne vide, on quitte l'itration. 
      if (($Ligne.trim() -eq "}") -or ($Ligne.trim() -eq "")) {break}
       # C'est une ligne de code, on l'insre 
      if ($Ligne.trim() -ne "{") 
       {
          # On rcupre le nom de la form dans $FormName
          # Note:  On recherche la ligne d'affectation du nom de la Form :  this.Name = "Form1";  
        if ($Ligne -match '^\s*this\.Name\s*=\s*"(?<nom>[^"]+)"\w*' ) 
          {$FormName = $matches["nom"]}

        [void]$Components.Add($Ligne)
        if ( $Ligne.contains("System.Windows.Forms.WebBrowser") )
         {Write-Host "Le composant WebBrowser n'est pas support sous PowerShell V1.0." -F Red; Finalyze; Return}
         if ( $Ligne.contains("System.ComponentModel.BackgroundWorker") )
         {Write-Warning "Les mthodes de thread du composant BackgroundWorker ne peuvent fonctionner sous PowerShell V1.0."}
       }
    }
}

if ($FormName -eq "") 
    {Throw "Le nom de la form est introuvable dans le fichier $Source.`n`rLa convertion ne peut s'effectuer." }

Write-Debug "Rcupration des lignes de code effectue."
# Collection de lignes constituant le nouveau script :  $LinesNewScript
# Note:
# Les dclarations des composants d'une Form se situent entre les lignes suivantes :
#
#   this.SuspendLayout();
#   ...
#   // Form 
#
# ----------------------------------------
$LinesNewScript = New-Object System.Collections.ArrayList(600)
[void]$LinesNewScript.Add( (Create-Header $Destination) )
[boolean] $IsTraiteMethodesForm = $False # Jusqu' la rencontre de la chane " # Form1  "
[boolean] $IsUsedResources= $false       # On utilise un fichier de ressources

 #Tableau de regex. Todo RegexCompilationInfo ?
$NbRegex=3
$LigneMultiple= [System.Array]::CreateInstance([string],$NbRegex,2) 
$Pattern=0; $SousPattern=1 #Fig
$Font=0; $Anchor=1;$ShortCutKeys=2;
     
$LigneMultiple[$Font,$Pattern]="\.Font = new System.Drawing.Font\("
$LigneMultiple[$Font,$SousPattern]="^\s\\| System.Drawing.FontStyle\."
     
$LigneMultiple[$Anchor,$Pattern]="\.Anchor = \(\(System.Windows.Forms.AnchorStyles\)"
$LigneMultiple[$Anchor,$SousPattern]="^\s\\| System.Windows.Forms.AnchorStyles\."

$LigneMultiple[$ShortcutKeys,$Pattern]="\.ShortcutKeys = \(\(System.Windows.Forms.Keys\)"
$LigneMultiple[$ShortCutKeys,$SousPattern]="^\s\\| System.Windows.Forms.Keys\."

#Recherche la prsence de dclarations d'une proprit sur plusieurs lignes :
for ($i=0; $i -le $Components.Count-1 ;$i++)
{    
    #Contrle la prsence d'un composant de gestion de ressources (images graphique principalement)
   if ($IsUsedResources -eq $false){
     $crMgr=[regex]::match($Components[$i],"\s= new System\.ComponentModel\.ComponentResourceManager\(typeof\((.*)\)\);$")
     if ($crMgr.success){
       $IsUsedResources = $True
       $Components[$i]=AjouteGestionRessources
       continue
     }
   }
  
    # Suppression des lignes contenant un appel aux mthodes suivantes : SuspendLayout, ResumeLayout et PerformLayout 
    #Ligne se terminant seulement par Layout(false); ou Layout(true); ou Layout();
   if ($Components[$i] -match ("Layout\((false|true)??\);$"))
     {$Components[$i]="";continue}
    
   if ($Components[$i].Contains("AutoScale"))
     {$Components[$i]="";Continue}

    # Aucun quivalent ne semble exister en Powershell pour ces commandes :
    # Pour les contrles : DataGridView, NumericUpDown et PictureBox
    # Suppression des lignes de gestion du DataBinding. 
   if ($Components[$i].Contains("((System.ComponentModel.ISupportInitialize)(" ))
     {$Components[$i]="";Continue}

   foreach($X in 0..($NbRegex-1))
   {      
     # La ligne est-elle une dclaration d'une proprit  vrifier ?
     #Ligne 1 :       this.NomComposant.NomPropritRecherch = dclaration spcifique
     $MatchTmp =[Regex]::Match($Components[$i],($LigneMultiple[$X,$Pattern]))
     if ($MatchTmp.Success -eq $true)
      { 
        #La ligne suivante est-elle un complment de cette dclaration
        #Ligne 2 :             | xxxxx)
        $MatchTmp =[Regex]::Match($Components[$i+1],($LigneMultiple[$X,$SousPattern]))
        if ($MatchTmp.Success -eq $true)
         { 
           $Components[$i]=$Components[$i]+$Components[$i+1].Trim()
           $Components[$i+1]=""
            #La ligne suivante est-elle aussi un complment de cette dclaration
            #Ligne 3 :             | XXXXX...
           $MatchTmp =[Regex]::Match($Components[$i+2],($LigneMultiple[$X,$SousPattern]))
           if ($MatchTmp.Success -eq $true)
            { 
	          $Components[$i]=$Components[$i]+$Components[$i+2].Trim()
              $Components[$i+2]=""
            } #If cas 3
          } #If cas 2
      } #If cas 1
   }#foreach
}#for
Write-Debug "Modifications des dclarations multi-lignes effectues."

if ($IsUsedResources -eq $true)
 { CompileRessources }

If(-not $DontLoad.ISPresent)
 {
    $Assemblies="System.Windows.Forms","System.Drawing"
    if ($IsUsedResources)
     {$Assemblies +="System.Resources"}
     
	[void]$LinesNewScript.Add("# Chargement des assemblies externes")
	 Load-Assembly $LinesNewScript $Assemblies
 }

 #Lance la modification du texte d'origine
foreach ($Ligne in $Components)
 {
    $Ligne = $Ligne.trim()
    if ($Ligne -eq "") {Continue} #Ligne suivante

     # On ajoute la cration d'un vnement
     # Gestion d'un event d'un composant :  this.btnRunClose.Click += new System.EventHandler(this.btnRunClose_Click);

      # La ligne dbute par "this" suivi d'un point
    if ($Ligne -match "^this\.?[^\.]+\.\w+ \+= new [A-Za-z0-9_\.]+EventHandler\(") 
     { 
        # On rcupre le nom du composant et le nom de l'vnement dans $T[1],$T[2]
       $T=$Ligne.Split(@(".","+"))
        #On ajoute le scriptblock grant l'vnement
       [void]$LinesNewScript.Add( (Create-EventComponent $T[1] $T[2].Trim()) )
       Continue
     }
        #Gestion d'un event de la form   : this.Load += new System.EventHandler(this.Form1_Load);
    elseif ($Ligne -match "^this\.?\w+ \+= new [A-Za-z0-9_\.]+EventHandler\(") 
      {
        # On rcupre le nom du composant et le nom de l'vnement dans $T[1],$T[2]
       $T=$Ligne.Split(@(".","+"))
       $EventName=$T[1].Trim()
        #On gnre par dfaut ces deux vnements
       if (($EventName -eq "FormClosing") -or ($EventName -eq "Shown")) {continue}
        #On ajoute le scriptblock grant l'vnement
       [void]$LinesNewScript.Add( (Create-EventComponent $FormName $EventName) )
       Continue
     }
      
# ------------ Traitement des lignes. Toutes ne sont pas encore supportes, i.e. correctement analyses

     #Recherche l'affectation d'une valeur d'numration par une capture
     # Trois groupe: 1- Les caractres  gauche de '= ', i.e. en dbut de ligne
     #				 2- Les caractres  droite de '= ' et avant le dernier '.'
     #				 3- Les caractres aprs le dernier '.'
     # Pour renforcer la reconnaissance on opre avant la suppression du ';'

     # On ne modifie pas les lignes du type :
     #       this.bindingNavigator1.AddNewItem = this.bindingNavigatorAddNewItem;
    $MatchTmp =[Regex]::Match($Ligne,"^.*= this.*")   
    if ($MatchTmp.Success -eq $false)
     {$Ligne = $Ligne -replace "^(.*)= (.*)\.(\w+);$", '$1=[$2]::$3'}

     # Suppression du token C# de fin de ligne 
    $Ligne = $Ligne -replace ";$",''

     # Suppression du token d'appel de mthode. ATTENTION. Utile uniquement pour les constructeurs !
    $Ligne = $Ligne -replace "\(\)$",''

     # Les lignes commentes le restent mais le traitement de la ligne courante se poursuit
    $Ligne = $Ligne -replace "^//",'#'
    
     # Remplacement des types boolean par les variables ddies 
    $Ligne = $Ligne -replace " true",' $true'
    $Ligne = $Ligne -replace " false",' $false'

     # Remplacement du format de typage des donnes
     #PB A quoi cela correspond-il ? si on remplace ici pb par la suite sur certaine ligne
     # A prioris le traitment n'est pas complet et fausse les analyses suivantes.
    #$Ligne = $Ligne -replace "\((\w+\.\w+\.\w+\.\w+)\)", '[$1]' 
     
      # Remplacement, dans le cadre du remplissage d'objets avec des valeurs, de 
      # la chane "new XXXXXX[] {" 
    $Ligne = $Ligne -replace "new [A-Za-z0-9_\.]+\[\] \{",'@('
     # Tjs dans le cadre du remplissage de listbox, remplacement de "})" par "))"
     #if ($Ligne.EndsWith("})")) {$Ligne = $Ligne.replace("})", '))')}
     $Ligne = $Ligne -replace "}\)$",'))'

#TODO : BUG dans la reconnaissance du pattern. Dcomposer la ligne qui peut tre complexe
#				  Saisie : "Test : &"''((--_)=+-*/.$^%,?;:~#{'[(-|`_\^@)]=}"
#				  C#     : "Test : &é\"\'\'((--èè_çà)=+-*/.$¨^%,?;:§~#{\'[(-|è`_\\ç^à@)]=}"});
#				  PS     : "Test : &\"\'\'((--_)=+-*/.$^%,?;:~#{\'[(--bor`_\\^@)]=}"))

     # si on trouve \'  entre 2 guillemets on le remplace par '
     # si on trouve \" entre 2 guillemets on le remplace par "
     # si on trouve \\ entre 2 guillemets on le remplace par \
     # si on trouve | entre 2 guillemets on ne le remplace pas
     # si on trouve || et qu'il n'est pas entre 2 guillemets on le remplace par -or (OR logique)

     # BUG : Remplacement de l'oprateur binaire OR
     #ATTENTION ne pas le modifier avant l'analyse des lignes de dclaration de fontes !!!
    #$ligne = $ligne.replace("|", '-bor')

     # Recherche dans les lignes commentes le nom de la form, 
     # le nombre d'espace entre # et Form1 importe peu mais il doit y en avoir au moins un.
    if ($Ligne -match "^#\s+" + $FormName) 
       {
        $IsTraiteMethodesForm = $True
         # On ajoute le constructeur de la Form
        [void]$LinesNewScript.Add("`$$FormName = new-object System.Windows.Forms.form")
         # Il n'existe qu'une ligne de ce type
        Continue 
       }
    if ($IsTraiteMethodesForm)
       {  # On modifie les chanes dbutant par "this"
          # Ex : "this.Controls.Add(this.maskedTextBox1) devient "$Form1.Controls.Add(this.maskedTextBox1)" 
         $Ligne = $Ligne -replace "^this.(.*)", "`$$FormName.`$1"
          # Ensuite on remplace toutes les occurences de "this". 
          # Ex :"$Form.Controls.Add(this.button1)" devient "$Form1.Controls.Add($button1)"         
         if ($Ligne.Contains("this."))  
          {$ligne = $Ligne.replace("this.", "$")}
       }
    else
       {  # On modifie les chanes dbutant par "this" qui oprent sur les composants
          # ,on remplace toutes les occurences de "this". 
          # Ex : "this.treeView1.TabIndex = 18" devient "$treeView1.TabIndex = 18" 
         if ($Ligne.StartsWith("this.")) 
           {$Ligne = $Ligne.replace("this.",'$')}
       }
      
      #Remplace le token d'appel d'un constructeur d'instance des composants graphique. 
      # this.PanelMainFill = new System.Windows.Forms.Panel();
    $Ligne = $Ligne.replace(" new ", " new-object ")
     #Todo this.tableLayoutPanelFill.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 50F));
    $ligne = $Ligne.replace("(new ", "(new-object ")
    
    $Ligne = $Ligne -replace "(^.*= new-object System.Drawing.SizeF\()([0-9]+)F, ([0-9]+)F\)$", '$1$2, $3)'
     #Traite les ressources images
    If ($IsUsedResources)
     {  #Todo : factoriser l'expression 
       $Ligne = $Ligne -replace "^(.*)= (\(\(System.Drawing.Image\)\(resources.GetObject\(`")(.*)`"\)\)\)$", '$1= [System.Drawing.Image] $Ressources["$3"]'
       $Ligne = $Ligne -replace "^(.*)= (\(\(System.Drawing.Icon\)\(resources.GetObject\(`")(.*)`"\)\)\)$", '$1= [System.Drawing.Icon] $Ressources["$3"]'
      }
     
# -------  Traite les proprits .Font
    $MatchTmp =[Regex]::Match($Ligne,'^(.*)(\.Font =.*System.Drawing.Font\()(.*)\)$')   
    if ($MatchTmp.Success -eq $true)
     { 
        #On traite la partie paramtres d'une dclaration 
       $ParametresFont = ParseProprieteFONT $MatchTmp
       $ligne = ReconstruitLigne $MatchTmp (1,2) $ParametresFont
       [void]$LinesNewScript.Add($ligne+")")
       continue
     }

# -------  Traite les proprits .Anchor
   #la ligne suivante est trait prcdement et ne match pas
   # this.button2.Anchor = System.Windows.Forms.AnchorStyles.None;
   #$button5.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)| System.Windows.Forms.AnchorStyles.Right)))
   $MatchTmp =[Regex]::Match($Ligne,'^(.*)(\.Anchor =.*System.Windows.Forms.AnchorStyles\))(.*)\)$')   
    if ($MatchTmp.Success -eq $true)
     { 
        #On traite la partie paramtres d'une dclaration 
       $ParametresAnchor = ParseProprieteANCHOR $MatchTmp
       $Ligne = ReconstruitLigne $MatchTmp (1) $ParametresAnchor
        #todo la function ReconstruitLigne est  revoir. Inadapt dans ce cas
        #$button1[System.Windows.Forms.AnchorStyles]"Top,Bottom,Left"
       $ligne = $ligne.replace("[System.",".Anchor = [System.")
       [void]$LinesNewScript.Add($Ligne)
       continue
     }

# -------  Traite les proprits .ShortcutKeys
   #$button5.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)| System.Windows.Forms.AnchorStyles.Right)))
   $MatchTmp =[Regex]::Match($Ligne,'^(.*)(\.ShortcutKeys = \(\(System.Windows.Forms.Keys\))(.*)\)$')   
    if ($MatchTmp.Success -eq $true)
     { 
        #On traite la partie paramtres d'une dclaration 
       $ParametresShortcutKeys = ParseProprieteSHORTcutKeys $MatchTmp
       $Ligne = ReconstruitLigne $MatchTmp (1) $ParametresShortcutKeys
        #todo la function ReconstruitLigne est  revoir. Inadapt dans ce cas
        #$button1[System.Windows.Forms.AnchorStyles]"Top,Bottom,Left"
       $ligne = $ligne.replace("[System.",".ShortcutKeys = [System.")
       [void]$LinesNewScript.Add($Ligne)
       continue
     }

# -------  Traite les appels de la mthode FormArgb
    $MatchTmp =[Regex]::Match($Ligne,'^(.*)( = System.Drawing.Color.FromArgb\()(.*)\)$') 
    if ($MatchTmp.Success -eq $true)
      { 
         #On traite la partie paramtres d'une dclaration 
        $ParametresRGB = ParseParametres_rgb $MatchTmp
        $Ligne = ReconstruitLigne $MatchTmp (1,2) $ParametresRGB
        $Ligne = $Ligne.Replace("System.Drawing.Color.FromArgb","[System.Drawing.Color]::FromArgb")
        [void]$LinesNewScript.Add($Ligne+")")
        continue
      }
# ------- Fertig !     
    [void]$LinesNewScript.Add($Ligne)
 }  # foreach
Write-Debug "Conversion du code CSharp effectue."

 [void]$LinesNewScript.Add( (Add-SpecialEventForm $FormName))
 
  # Pas d'appel  la mthode ShowDialog
 If( -not $dontshow.ISPresent )
  { [void]$LinesNewScript.Add("`$$FormName.ShowDialog()") }

 [void]$LinesNewScript.Add(" #Libration des ressources")
 If ($IsUsedResources)
  { [void]$LinesNewScript.Add("`$Reader.Close()  #Appel Dispose") }
 [void]$LinesNewScript.Add("`$$FormName.Dispose()")

  # Ecriture du fichier de sortie
 trap [System.UnauthorizedAccessException]
 { Write-Host $Error[0].Exception.Message -F Red; continue} 
 &{ 
    $LinesNewScript | Out-File -FilePath $Destination -Encoding Default
     # Vrifie la syntaxe du script gnr
    CheckSyntaxErrors $Destination -verbose|Out-null
 }
 
 Finalyze
 