# ==============================================================================================
# 
# 
# Nom: PackageAdvancedDotNEt.ps1
# 
# Date  : 26/03/2008
# 
# Commentaire : Contient des fonctions avances pour dotNET
#			-Cration de dlgu
#			-Appel de code Win32
#			-Compilation  la vole de code CSharp
# 
# ==============================================================================================
function Get-Delegate([type] $type, [ScriptBlock] $scriptBlock)
{
 #Obtient, aprs sa cration  la vole, un dlgu .NET
 #Par Bruce Payette :
 # http://blogs.msdn.com/powershell/archive/2006/07/25/678259.aspx
 #
 #Exemple d'appel :
 # $Delegate = Get-Delegate System.Text.RegularExpressions.MatchEvaluator {Traitement}
 #
 # $Expression = [regex] '^(.*)(\.Font =.*System.Drawing.Font\()(.*)\)$'
 # $Expression.Replace($ligne, $Delegate)

# Helper function to emit an IL opcode
function emit($opcode)
{
    if ( ! ($op = [System.Reflection.Emit.OpCodes]::($opcode)))
    {
        throw "new-method: opcode '$opcode' is undefined"
    }

    if ($args.Length -gt 0)
    {
        $ilg.Emit($op, $args[0])
    }
    else
    {
        $ilg.Emit($op)
    }
}

	# Get the method info for this delegate invoke...
	$delegateInvoke = $type.GetMethod("Invoke")
	
	# Get the argument type signature for the delegate invoke
	$parameters = @($delegateInvoke.GetParameters())
	$returnType = $delegateInvoke.ReturnParameter.ParameterType
	
	$argList = new-object Collections.ArrayList
	[void] $argList.Add([ScriptBlock])
	foreach ($p in $parameters)
	{
	    [void] $argList.Add($p.ParameterType);
	}
	
	$dynMethod = new-object reflection.emit.dynamicmethod ("",
	    $returnType, $argList.ToArray(), [object], $false)
	$ilg = $dynMethod.GetILGenerator()
	
	# Place the scriptblock on the stack for the method call
	emit Ldarg_0
	
	emit Ldc_I4 ($argList.Count - 1)  # Create the parameter array
	emit Newarr ([object])
	
	for ($opCount = 1; $opCount -lt $argList.Count; $opCount++)
	{
	    emit Dup                    # Dup the array reference
	    emit Ldc_I4 ($opCount - 1); # Load the index
	    emit Ldarg $opCount         # Load the argument
	    if ($argList[$opCount].IsValueType) # Box if necessary
		{
	        emit Box
		}
	    emit Stelem ([object])  # Store it in the array
	}
	
	# Now emit the call to the ScriptBlock invoke method
	emit Call ([ScriptBlock].GetMethod("InvokeReturnAsIs"))
	
	if ($returnType -eq [void])
	{
	    # If the return type is void, pop the returned object
	    emit Pop
	}
	else
	{
	    # Otherwise emit code to convert the result type which looks
	    # like LanguagePrimitives.ConvertTo(value, type)
	
	    $signature = [object], [type]
	    $convertMethod =
	        [Management.Automation.LanguagePrimitives].GetMethod(
	            "ConvertTo", $signature);
	    $GetTypeFromHandle = [Type].GetMethod("GetTypeFromHandle");
	    emit Ldtoken $returnType  # And the return type token...
	    emit Call $GetTypeFromHandle
	    emit Call $convertMethod
	}
	emit Ret
	
#
# Now return a delegate from this dynamic method...
#
	
   $dynMethod.CreateDelegate($type, $scriptBlock)
}


function Invoke-Win32([string] $dllName, [Type] $returnType,  
   [string] $methodName, [Type[]] $parameterTypes, [Object[]] $parameters) 
{ # Appel du code Win32 natif via le systme P/Invoke de .NET . 
  # Par Lee Holmes : 
  #  http://www.leeholmes.com/blog/GetTheOwnerOfAProcessInPowerShellPInvokeAndRefOutParameters.aspx
  #
  #Exemple d'appel : Voir l'URL cite
 
   ## Begin to build the dynamic assembly 
   $domain = [AppDomain]::CurrentDomain 
   $name = New-Object Reflection.AssemblyName 'PInvokeAssembly' 
   $assembly = $domain.DefineDynamicAssembly($name, 'Run') 
   $module = $assembly.DefineDynamicModule('PInvokeModule') 
   $type = $module.DefineType('PInvokeType', "Public,BeforeFieldInit") 

   ## Go through all of the parameters passed to us.  As we do this, 
   ## we clone the user's inputs into another array that we will use for 
   ## the P/Invoke call.   
   $inputParameters = @() 
   $refParameters = @() 
   
   for($counter = 1; $counter -le $parameterTypes.Length; $counter++) 
   { 
      ## If an item is a PSReference, then the user  
      ## wants an [out] parameter. 
      if($parameterTypes[$counter - 1] -eq [Ref]) 
      { 
         ## Remember which parameters are used for [Out] parameters 
         $refParameters += $counter 

         ## On the cloned array, we replace the PSReference type with the  
         ## .Net reference type that represents the value of the PSReference,  
         ## and the value with the value held by the PSReference. 
         $parameterTypes[$counter - 1] =  
            $parameters[$counter - 1].Value.GetType().MakeByRefType() 
         $inputParameters += $parameters[$counter - 1].Value 
      } 
      else 
      { 
         ## Otherwise, just add their actual parameter to the 
         ## input array. 
         $inputParameters += $parameters[$counter - 1] 
      } 
   } 

   ## Define the actual P/Invoke method, adding the [Out] 
   ## attribute for any parameters that were originally [Ref]  
   ## parameters. 
   $method = $type.DefineMethod($methodName, 'Public,HideBySig,Static,PinvokeImpl',  
      $returnType, $parameterTypes) 
   foreach($refParameter in $refParameters) 
   { 
      $method.DefineParameter($refParameter, "Out", $null) 
   } 

   ## Apply the P/Invoke constructor 
   $ctor = [Runtime.InteropServices.DllImportAttribute].GetConstructor([string]) 
   $attr = New-Object Reflection.Emit.CustomAttributeBuilder $ctor, $dllName 
   $method.SetCustomAttribute($attr) 

   ## Create the temporary type, and invoke the method. 
   $realType = $type.CreateType() 
   $realType.InvokeMember($methodName, 'Public,Static,InvokeMethod', $null, $null,  
      $inputParameters) 

   ## Finally, go through all of the reference parameters, and update the 
   ## values of the PSReference objects that the user passed in. 
   foreach($refParameter in $refParameters) 
   { 
      $parameters[$refParameter - 1].Value = $inputParameters[$refParameter - 1] 
   } 
} 

function Compile-Csharp ([string] $code, $FrameworkVersion="v2.0.50727", 
[Array]$References)
{ #Compile  la vole du code CSharp un assembly
  # par J.Snover :
  #  http://blogs.msdn.com/powershell/archive/2006/04/25/583236.aspx
  
    #
    # Get an instance of the CSharp code provider
    #
    $cp = new-object Microsoft.CSharp.CSharpCodeProvider

    #
    # Build up a compiler params object...
    $framework = Combine-Path $env:windir "Microsoft.NET\Framework\$FrameWorkVersion"
    $refs = new Collections.ArrayList
    $refs.AddRange( @("${framework}\System.dll",
        "${mshhome}\System.Management.Automation.dll",
        "${mshhome}\System.Management.Automation.ConsoleHost.dll",
        "${framework}\system.windows.forms.dll",
        "${framework}\System.data.dll",
        "${framework}\System.Drawing.dll",
        "${framework}\System.Xml.dll"))
    if ($references.Count -ge 1)
    {
        $refs.AddRange($References)
    }

    $cpar = New-Object System.CodeDom.Compiler.CompilerParameters
    $cpar.GenerateInMemory = $true
    $cpar.GenerateExecutable = $false
    $cpar.OutputAssembly = "custom"
    $cpar.ReferencedAssemblies.AddRange($refs)
    $cr = $cp.CompileAssemblyFromSource($cpar, $code)

    if ( $cr.Errors.Count)
    {
        $codeLines = $code.Split("`n");
        foreach ($ce in $cr.Errors)
        {
            write-host "Error: $($codeLines[$($ce.Line - 1)])"
            $ce |out-default
        }
        Throw "INVALID DATA: Errors encountered while compiling code"
    }
}
