################################################################################
#                    www.PowerShell-Scripting.com
#
#                      PowerShell Path package
#
# Version : 0.2
#
# Statut  : A valider
#
# Date    : 21 mars 2008
#
# Nom     : PackageScripts.ps1
#
# Usage   : . .\PackageScripts.ps1
#
# Objet   : Regroupe une srie de fonctions manipulant l'environnement d'appel d'un script
# 
# Limitations :
#
# Todo    : 
#       La fonction IsAssemblyLoaded  est  test
#       La fonction  IsStrongNameAssemblyLoaded n'est pas implment 
#
##########################################################################

function Get-ScriptDirectory
{  #Renvoi le nom du rpertoire d'un script parent, celui appel sur la ligne de commande.
   # By J.Snover
   #Attention cette function doit tre copi dans le script principal
  $Invocation = (Get-Variable MyInvocation -Scope 1).Value
  Split-Path $Invocation.MyCommand.Path
}

function Get-ScriptName
{  #Renvoi le nom d'un script parent sans son extension.
  [System.IO.Path]::GetFilenameWithoutExtension( (Get-ScriptDirectory) )
}

function local:TestRequiredItem([String[]] $required, [string] $NomItem )
{  #D'aprs une ide de J.Snover
   # Test si les lments rfrences(Cmdlet, fonction, script) dans le tableau $required existent
   #C'est en quelque sorte un contrle des prrequis  l'excution d'un script
  if (($required -eq $null) -or ($required.count -eq 0))
    { throw "Le tableau contenant les noms d'lments  rechercher ne peut tre null ou vide."}
  switch ($NomItem)
  {
    "Commande" {$null=Get-Command $required -ea SilentlyContinue -ev missingRequired
    			for($I=0; $I -le $missingRequired.count -1; $i++){
	 				Write-Warning "$NomItem introuvable $($missingRequired[$i].Exception.CommandName)"
  				} 
  				if ($missingRequired.Count) 
      			 #$($NomItem) = concatenation de variable et d'un ou + caractres
   				{ throw "$($NomItem)s introuvables. Le traitement ne peut continuer." }
                break}
     #Pour Test-path missingRequired est renseign uniquement avec des erreurs irrcuprables
    "Fonction" {$required|% {if ((Test-Path function:$_ -ea SilentlyContinue -ev missingRequired) -eq $false) {[array]$missingPath +=$_}}
    			Break}
    "Script"   {$required|% {if ((Test-Path $_ -ea SilentlyContinue -ev missingRequired) -eq $false) {[array]$missingPath +=$_}}
    			break}
    Default    {throw "Cas imprvu : $NomItem"}
  }
 
  for($I=0; $I -le $missingPath.count-1; $i++) {
	 Write-Warning "$NomItem introuvable $($missingPath[$i])"
  } 
  if (($missingRequired.Count -gt 0 ) -or ($missingPath.Count -gt 0))
      #$($Item) = concatnation de variable et d'un ou + caractres
   { throw "$($NomItem)s introuvables. Le traitement ne peut continuer." }
}

function TestRequiredCommands([String[]] $requiredCommands)
{  #Nom des item requis: TestRequiredCommands "Test-MyData", "Add-MyData"
  TestRequiredItem $requiredCommands "Commande"
}

function TestRequiredScripts([String[]] $requiredScripts)
{ #Nom des item requis: TestRequiredScripts "c:\PS\Scripts\PackageWinform.ps1"
  TestRequiredItem $requiredScripts "Script"
}

function TestRequiredFunctions([String[]] $requiredFunctions)
{ #Nom des item requis: TestRequiredFunctions "c:\PS\Scripts\PackageWinform.ps1"
  TestRequiredItem $requiredFunctions "Fonction"
}

Function CheckSyntaxErrors($path, [switch]$verbose) 
{ 
 # Vrifie la syntaxe d'un script mais pas sa cohrence
 #  http://keithhill.spaces.live.com/blog/cns!5A8D2641E0963A97!6036.entry

 if (!$path){
    Write-Host "Usage :`n`r CheckSyntaxErrors.ps1 NomDeFichier.ps1`n`r"
    Write-Host " Vrifie la syntaxe d'un script. L'argument -verbose spcifie l'affiche des informations supplmentaires lors de la vrification."
    Return
  }   

 if ($verbose) {
    $VerbosePreference = 'Continue'
 } 

 trap { Write-Warning $_; $false; continue }
 & `
 {
    if (-not(Test-Path $Path)) {Throw "Le fichier n'existe pas : $path"}
    $contents = get-content $Path
    $contents = [string]::Join([Environment]::NewLine, $contents)
    [void]$ExecutionContext.InvokeCommand.NewScriptBlock($contents)
    Write-Host "La syntaxe du script est correcte. Contrler la validit de ses instructions." -F Green
    $true
 } 
}

function IsAssemblyLoaded([string] $NomAssembly, [switch] $LoadAssembly){    
#Charge un assembly d'aprs son nom.
  if ( ([appdomain]::CurrentDomain.GetAssemblies()|where { $_.FullName -match "^$($NomAssembly),"}) -ne $null )
   {
     if ($LoadedAssembly.IsPresent)
      {
        [void][Reflection.Assembly]::LoadWithPartialName("$NomAssembly")
        return $true
      }
   }
  return $false
}  

function IsStrongNameAssemblyLoaded([string] $NomAssembly,[switch] $LoadAssembly){    
#Charge un assembly possdant un nom fort.
#todo Charger d'aprs un chemin complet 
  Throw "La fonction IsStrongNameAssemblyLoaded n'est pas implment."
}  
